#!/bin/ksh
################################################
#
# QuickBridge.ksh - a wrapper for the bridge software
#
# Author: Michael Daw
#         (c) University of Manchester 2002
#
# Version: 2.0
#
# Change history:
#    1.1 Added flags to enable forcing ports
#    2.0 Venues information taken from data file
#
# Please send all bug reports to: michael.daw@man.ac.uk
#
################################################

################################################
# Function
################################################

function Usage
{
    echo "Usage: QuickBridge.ksh -r virtualvenue [-s|-k] [options]"
    echo ""
    echo "Mandatory parameters:"
    echo "  -r virtualvenue        Specifies Virtual Venue"
    echo "                         (works on short name specified in vv_file)"
    echo "  -s                     Starts QuickBridge"
    echo "  -k                     Stops (kills) QuickBridge"
    echo ""
    echo "Options:"
    echo "  -f vv_file             Used to specify Virtual Venue file"
    echo "                         (defaults to './vv.dat')"
    echo "                         For file format, specify '-f help'"
    echo "  -a audio_unicast_port  Used with -s option"
    echo "                         Force audio to be on specified port"
    echo "                         Must be different from video port, even and >=50000"
    echo "  -v video_unicast_port  Used with -s option"
    echo "                         Force video to be on specified port"
    echo "                         Must be different from video port, even and >=50000"

    exit
}

################################################
# Function

function FileUsage
{
    echo "Virtual Venues data file format"
    echo "-------------------------------"
    echo "  Data file name defaults to './vv.dat' if -f option not specified"
    echo "  Format:"
    echo "  <example file begins>"
    echo "  # Comment has '#' at start of line"
    echo "  # ShortName a/v McastAdd McastPort UcastPort LongName"
    echo "  Ful v 224.2.132.254 51258 50002 Full Sail Room"
    echo "  Ful a 224.2.215.244 21288 50004 Full Sail Room"
    echo "  <example file ends>"

    exit
}

################################################
# Function

function KillProcess
{
    # Get the process id
    bridge_ppid=`ps -ef | grep -v 'awk' | awk '/'$mcast_add'/ {print $2}'`

    # Kill them
    if [ -n "${bridge_ppid}" ]
    then
        kill ${bridge_ppid} > /dev/null
        sleep 1
    else
        return 1
    fi

    # Check it's really gone
    bridge_ppid=`ps -ef | grep -v 'awk' | awk '/'$mcast_add'/ {print $2}'`

    if [ -z "${bridge_ppid}" ]
    then
        return 0
    else
        return -1
    fi
}

################################################
# Main control
################################################

# Initialise and check variables/parameters
bridge_exe=QuickBridge
ttl=127
vv_file=vv.dat

while [[ $# -ne 0 ]]
do
    case ${1} in 
    (-r)
        if [[ -n ${2} ]]
        then
            shift
            vv=${1}
            shift
        else
            Usage
        fi
        ;;
    (-s)
        shift
        if [[ -n ${startkill} ]]
        then
            Usage
        fi
        startkill="start"
        ;;
    (-k)
        shift
        if [[ -n ${startkill} ]]
        then
            Usage
        fi
        startkill="kill"
        ;;
    (-a)
        if [[ -n ${2} ]]
        then
            shift
            aud_ucast_port=${1}
            shift
        else
            Usage
        fi
        ;;
    (-v)
        if [[ -n ${2} ]]
        then
            shift
            vid_ucast_port=${1}
            shift
        else
            Usage
        fi
        ;;
    (-f)
        if [[ -n ${2} ]]
        then
            shift
            vv_file=${1}
            shift
        else
            Usage
        fi
        if [[ $vv_file == "help" ]]
        then
            FileUsage
        fi
        ;;
    (*)
        Usage
        ;;
    esac
done

# Does file exist?
if [[ ! -f $vv_file ]]
then
    echo "Error: Virtual venue file '$vv_file' does not exist"
    Usage
fi

# Can't go on without venue or start/kill
if [[ -z ${vv} || -z ${startkill} ]]
then
    Usage
fi

# Read parameters from file
while read short_name a_v mcast_add mcast_port ucast_port long_name
do
    if [[ $short_name != '#'* ]]
    then
        if [[ $short_name != $vv ]]
        then
            continue
        fi

        case $a_v in
        ("v")
            vid_mcast_port=$mcast_port
            vid_mcast_add=$mcast_add

            if [[ -z ${vid_ucast_port} ]]
            then
                vid_ucast_port=$ucast_port
            fi
            ;;
        ("a")
            aud_mcast_port=$mcast_port
            aud_mcast_add=$mcast_add

            if [[ -z ${aud_ucast_port} ]]
            then
                aud_ucast_port=$ucast_port
            fi
            ;;
        (*)
            FileUsage
            ;;
        esac
        
        vv_name=$long_name
    fi
done < $vv_file

# Check we found a venue
if [[ -z $vv_name ]]
then
    echo "Short name '$vv' not found in file '$vv_file' or entry does not have long name"
    echo ""
    FileUsage
fi

# Can't have the same audio / video ports
if [[ ${aud_ucast_port} -eq ${vid_ucast_port} ]]
then
    Usage
fi

# Start or kill the bridge
case ${startkill} in
(start)
    echo "Starting bridge for ${vv_name}..."

    ${bridge_exe} -g ${vid_mcast_add} -m ${vid_mcast_port} \
        -u ${vid_ucast_port} -t ${ttl} &

    ${bridge_exe} -g ${aud_mcast_add} -m ${aud_mcast_port} \
        -u ${aud_ucast_port} -t ${ttl} &

    echo ""
    echo "################################################"
    echo "Audio for ${vv_name} is on port ${aud_ucast_port}"
    echo "Video for ${vv_name} is on port ${vid_ucast_port}"
    echo "################################################"
    echo ""

    exit
    ;;

(kill)
    echo "Stopping bridge for ${vv_name}..."

    for mcast_add in $vid_mcast_add $aud_mcast_add
    do
        KillProcess

        case $? in
        (0)
            echo "Bridge for mcast address '$mcast_add' killed successfully"
            ;;
        (1)
            echo "Warning: no bridge for mcast address '$mcast_add' to kill"
            ;;
        (-1)
            echo "Error: bridge for mcast address '$mcast_add' NOT killed!"
            ;;
        esac
    done

    exit
    ;;
esac

# Script ends
